
local function debug(msg)
	Soundtrack.Util.DebugPrint(msg, 0.6, 0.6, 1.0)
end

Soundtrack.CustomEvents = {
    BuffEvents = {}
}

Soundtrack.CombatLogDelayTime = 0
Soundtrack.CombatLogUpdateTime = .1
Soundtrack.ActionHouse = false
Soundtrack.Bank = false
Soundtrack.Merchant = false

Soundtrack.CurrentStance = 0;
Soundtrack.TreeForm = false;
Soundtrack.MoonkinForm = false;

function Soundtrack.CustomEvents.RegisterTrigger(trigger)
	if trigger ~= nil then
		this:RegisterEvent(trigger);
	end
end

-- table can be Custom or Misc. 
function Soundtrack.CustomEvents.RegisterUpdateScript(name, tableName, _priority, _continuous, _script)
    if tableName == "Custom" then
        Soundtrack_CustomEvents[name] = { script = _script, eventtype = "Update Script", priority=_priority, continuous=_continuous };
    elseif tableName == "Misc" then
        Soundtrack_MiscEvents[name] = { script = _script, eventtype = "Update Script", priority=_priority, continuous=_continuous };
    end
    
    if _trigger ~= nil then
        this:RegisterEvent(_trigger);
    end
    
    Soundtrack.AddEvent(tableName, name, _priority, _continuous)
end

-- table can be Custom or Misc. 
function Soundtrack.CustomEvents.RegisterEventScript(name, tableName, _trigger, _priority, _continuous, _script)
    if tableName == "Custom" then
        Soundtrack_CustomEvents[name] = { trigger = _trigger, script = _script, eventtype = "Event Script", priority=_priority, continuous=_continuous };
    elseif tableName == "Misc" then
        Soundtrack_MiscEvents[name] = { trigger = _trigger, script = _script, eventtype = "Event Script", priority=_priority, continuous=_continuous }; 
    end
    Soundtrack.CustomEvents.RegisterTrigger(_trigger)
    
    Soundtrack.AddEvent(tableName, name, _priority, _continuous)
end

function Soundtrack.IsBuffActive(buffTexture)
	if buffTexture == nil then
		return false
	end
    for i=1,40 do
		local n, _, icon, _, _, _, _, _, _, _, spellId = UnitBuff("player", i); 
		if icon == nil then
			return false
		elseif string.lower(icon) == string.lower(buffTexture) then
			--Soundtrack.Trace("Buff: "..string.lower(icon) .. " " .. spellId)
	        return spellId
	    end
	end
	return nil
end
function Soundtrack.IsDebuffActive(buffTexture)
	if buffTexture == nil then
		return false
	end
    for i=1,40 do
		local n, _, icon, _, _, _, _, _, _, _, spellId = UnitDebuff("player", i); 
	    if icon == nil then
			return false
		elseif string.lower(icon) == string.lower(buffTexture) then
			--Soundtrack.Trace("Debuff: "..string.lower(icon) .. " " .. spellId)
	        return spellId
	    end
	end
	return nil
end

function Soundtrack.CustomEvents.RegisterBuffEvent(eventName, tableName, _buffTexture, _priority, _continuous)
    if tableName == "Custom" then
        Soundtrack_CustomEvents[eventName] = 
        { 
            buffTexture=_buffTexture, 
            stackLevel=_priority, 
            active=false,
            eventtype="Buff",
            priority=_priority, 
            continuous=_continuous
        }    
    elseif tableName == "Misc" then
        Soundtrack_MiscEvents[eventName] = 
        { 
            buffTexture=_buffTexture, 
            stackLevel=_priority, 
            active=false,
            eventtype="Buff",
            priority=_priority, 
            continuous=_continuous
        }  
    end
    Soundtrack.AddEvent(tableName, eventName, _priority, _continuous);
end

function Soundtrack_Custom_PlayEvent(tableName, eventName)
	local eventTable = Soundtrack.Events.GetTable(tableName)
	local currentEvent = Soundtrack.Events.Stack[eventTable[eventName].priority].eventName
	if eventName == currentEvent then return end -- Don't do anything.  Event already playing.
	
	if eventTable[eventName].trigger == "COMBAT_LOG_EVENT_UNFILTERED" then
		local currentTime = GetTime()
		if currentTime >= Soundtrack.CombatLogDelayTime then
			Soundtrack.CombatLogDelayTime = currentTime + Soundtrack.CombatLogUpdateTime
			Soundtrack.PlayEvent(tableName, eventName);
		end
	else
		Soundtrack.PlayEvent(tableName, eventName);
	end
end
function Soundtrack_Custom_StopEvent(tableName, eventName)
	--[[local eventTable = Soundtrack.Events.GetTable(tableName)
	if eventTable[eventName] then
		local trackList = eventTable[eventName].tracks
		if trackList then
			local numTracks = table.getn(trackList)
			if numTracks >= 1 then
				Soundtrack.StopEvent(tableName, eventName);
			end
		end
	end  -- Track check in main code to improve performance.
	--]]
	local eventTable = Soundtrack.Events.GetTable(tableName)
	local currentEvent = Soundtrack.Events.Stack[eventTable[eventName].priority].eventName
	if eventName ~= currentEvent then return end -- Don't do anything.  Event not playing.
	
	Soundtrack.StopEvent(tableName, eventName);
end

-- Make sure that tableName and eventName exist.
-- Function does not check if they do.
local function Soundtrack_Custom_HasTracks(tableName, eventName)
	local eventTable = Soundtrack.Events.GetTable(tableName)
	local numTracks = table.getn(eventTable[eventName].tracks)
	if numTracks >= 1 then
		return true
	else
		return false
	end
end

local jumpDelayTime = 0;
local jumpUpdateTime = .7

hooksecurefunc("JumpOrAscendStart",
	function()
		if HasFullControl() and not IsFlying() and not IsSwimming() and not UnitInVehicle("player") then 
			local currentTime = GetTime()
			if currentTime >= jumpDelayTime then
				jumpDelayTime = currentTime + jumpUpdateTime;
				Soundtrack.Trace("Jumping!")
				Soundtrack_Custom_PlayEvent("Misc", SOUNDTRACK_JUMP)
			end
		end
	end
);

function Soundtrack.CustomEvents.Initialize()
    -- this:RegisterEvent("PLAYER_AURAS_CHANGED") -- Replaced by UNIT_AURA
	this:RegisterEvent("UNIT_AURA")
    this:RegisterEvent("AUCTION_HOUSE_CLOSED")
    this:RegisterEvent("AUCTION_HOUSE_SHOW")
    this:RegisterEvent("BANKFRAME_CLOSED")
    this:RegisterEvent("BANKFRAME_OPENED")
    this:RegisterEvent("MERCHANT_CLOSED")
    this:RegisterEvent("MERCHANT_SHOW")
    
    
	
    -- Add fixed custom events 
	-- Jump
	Soundtrack.CustomEvents.RegisterUpdateScript(
        SOUNDTRACK_JUMP, 
        "Misc",
        8,
        false,
        function()
			-- hooksecurefunc for "JumpOrAscendStart"
		end
	);
	-- Swimming
    Soundtrack.CustomEvents.RegisterUpdateScript(
        SOUNDTRACK_SWIMMING, 
        "Misc",
        5,
        true,
        function()
            if SNDCUSTOM_IsSwimming == nil then 
                SNDCUSTOM_IsSwimming = false 
            end

            if SNDCUSTOM_IsSwimming and not IsSwimming() then
				Soundtrack_Custom_StopEvent("Misc", SOUNDTRACK_SWIMMING)
                SNDCUSTOM_IsSwimming = false
            elseif not SNDCUSTOM_IsSwimming and IsSwimming() then
                Soundtrack_Custom_PlayEvent("Misc", SOUNDTRACK_SWIMMING)
                SNDCUSTOM_IsSwimming = true;
            end
        end
	);
	-- Auction House  
	Soundtrack.CustomEvents.RegisterUpdateScript(
        SOUNDTRACK_AUCTION_HOUSE, 
        "Misc",
        5,
        true,
        function()
            if SNDCUSTOM_AuctionHouse == nil then 
                SNDCUSTOM_AuctionHouse = false 
            end

            if SNDCUSTOM_AuctionHouse and not Soundtrack.AuctionHouse then
                Soundtrack_Custom_StopEvent("Misc", SOUNDTRACK_AUCTION_HOUSE)
                SNDCUSTOM_AuctionHouse = false
            elseif not SNDCUSTOM_AuctionHouse and Soundtrack.AuctionHouse then
                Soundtrack_Custom_PlayEvent("Misc", SOUNDTRACK_AUCTION_HOUSE)
                SNDCUSTOM_AuctionHouse = true;
            end
        end
	    );
	-- Bank  
	Soundtrack.CustomEvents.RegisterUpdateScript(
        SOUNDTRACK_BANK, 
        "Misc",
        5,
        true,
        function()
            if SNDCUSTOM_Bank == nil then 
                SNDCUSTOM_Bank = false 
            end

			-- Soundtrack.* does not deal with localization
            if SNDCUSTOM_Bank and not Soundtrack.Bank then
                Soundtrack_Custom_StopEvent("Misc", SOUNDTRACK_BANK)
                SNDCUSTOM_Bank = false
            elseif not SNDCUSTOM_Bank and Soundtrack.Bank then
                Soundtrack_Custom_PlayEvent("Misc", SOUNDTRACK_BANK)
                SNDCUSTOM_Bank = true;
            end
        end
	    );
	-- Merchant   
	Soundtrack.CustomEvents.RegisterUpdateScript(
        SOUNDTRACK_MERCHANT, 
        "Misc",
        5, -- TODO Anthony: This conflicts with battle level, was 6
        true,
        function()
            if SNDCUSTOM_Merchant == nil then 
                SNDCUSTOM_Merchant = false 
            end
			
            if SNDCUSTOM_Merchant and not Soundtrack.Merchant then
                Soundtrack_Custom_StopEvent("Misc", SOUNDTRACK_MERCHANT)
                SNDCUSTOM_Merchant = false
            elseif not SNDCUSTOM_Merchant and Soundtrack.Merchant then
                Soundtrack_Custom_PlayEvent("Misc", SOUNDTRACK_MERCHANT)
                SNDCUSTOM_Merchant = true;
            end
        end
	    );	    
	-- Level Up
	Soundtrack.CustomEvents.RegisterEventScript(
	    SOUNDTRACK_LEVEL_UP,
	    "Misc",
	    "PLAYER_LEVEL_UP",
	    8,
	    false,
	    function()
	        Soundtrack_Custom_PlayEvent("Misc", SOUNDTRACK_LEVEL_UP);
	    end
	    );
	-- Join Party
	Soundtrack.CustomEvents.RegisterEventScript(
	    SOUNDTRACK_JOIN_PARTY,
	    "Misc",
	    "PARTY_MEMBERS_CHANGED",
	    6,
	    false,
	    function()
	        if SOUNDTRACK_InParty == nil then 
                SOUNDTRACK_InParty = false 
            end
	        if not SOUNDTRACK_InParty and GetNumRaidMembers() == 0 and GetNumPartyMembers() > 0 then
				Soundtrack_Custom_PlayEvent("Misc", SOUNDTRACK_JOIN_PARTY)
                SOUNDTRACK_InParty = true
	        elseif SOUNDTRACK_InParty and GetNumPartyMembers() == 0 then
                SOUNDTRACK_InParty = false
	        end
	    end
	    );
	-- Join Raid  
	Soundtrack.CustomEvents.RegisterEventScript(
	    SOUNDTRACK_JOIN_RAID,
	    "Misc",
	    "PARTY_MEMBERS_CHANGED",
	    6,
	    false,
	    function()
	        if SOUNDTRACK_InRaid == nil then 
                SOUNDTRACK_InRaid = false 
            end
	        if not SOUNDTRACK_InRaid and GetNumRaidMembers() > 0 then
                Soundtrack_Custom_PlayEvent("Misc", SOUNDTRACK_JOIN_RAID)
                SOUNDTRACK_InRaid = true
	        elseif SOUNDTRACK_InRaid and not GetNumRaidMembers() == 0 then
                SOUNDTRACK_InRaid = false
	        end
	    end
	);
	
	-- Death Knight Change Presence
	Soundtrack.CustomEvents.RegisterEventScript(
	    SOUNDTRACK_DK_CHANGE,
	    "Misc",
	    "UPDATE_SHAPESHIFT_FORM",
	    8,
	    false,
	    function()
	        local class = UnitClass("player")
			local stance = GetShapeshiftForm();
			if class == "Death Knight" and stance ~= 0 and stance ~= Soundtrack.CurrentStance then
				Soundtrack_Custom_PlayEvent("Misc", SOUNDTRACK_DK_CHANGE)
			end
			Soundtrack.CurrentStance = stance;
	    end
	);
	-- Druid Change Form
	Soundtrack.CustomEvents.RegisterEventScript(
	    SOUNDTRACK_DRUID_CHANGE,
	    "Misc",
	    "UPDATE_SHAPESHIFT_FORM",
	    8,
	    false,
	    function()
	        local class = UnitClass("player")
			local stance = GetShapeshiftForm();
			if class == "Druid" and stance ~= 0 and stance ~= Soundtrack.CurrentStance then
				Soundtrack_Custom_PlayEvent("Misc", SOUNDTRACK_DRUID_CHANGE)
			end
			Soundtrack.CurrentStance = stance;
	    end
	);
	-- Paladin Change Auras
	Soundtrack.CustomEvents.RegisterEventScript(
	    SOUNDTRACK_PALADIN_CHANGE,
	    "Misc",
	    "UPDATE_SHAPESHIFT_FORM",
	    8,
	    false,
	    function()
	        local class = UnitClass("player")
			local stance = GetShapeshiftForm();
			if class == "Paladin" and stance ~= 0 and stance ~= Soundtrack.CurrentStance then
				Soundtrack_Custom_PlayEvent("Misc", SOUNDTRACK_PALADIN_CHANGE)
			end
			Soundtrack.CurrentStance = stance;
	    end
	);
	-- Priest Change Shadowform
	Soundtrack.CustomEvents.RegisterEventScript(
	    SOUNDTRACK_PRIEST_CHANGE,
	    "Misc",
	    "UPDATE_SHAPESHIFT_FORM",
	    8,
	    false,
	    function()
	        local class = UnitClass("player")
			local stance = GetShapeshiftForm();
			if class == "Priest" and stance ~= 0 then
				Soundtrack_Custom_PlayEvent("Misc", SOUNDTRACK_PRIEST_CHANGE)
			end
			Soundtrack.CurrentStance = stance;
	    end
	);
	-- Rogue Change Stealth
	Soundtrack.CustomEvents.RegisterEventScript(
	    SOUNDTRACK_ROGUE_CHANGE,
	    "Misc",
	    "UPDATE_SHAPESHIFT_FORM",
	    8,
	    false,
	    function()
	        local class = UnitClass("player")
			local stance = GetShapeshiftForm();
			if class == "Rogue" and stance ~= 0 then
				Soundtrack_Custom_PlayEvent("Misc", SOUNDTRACK_ROGUE_CHANGE)
			end
			Soundtrack.CurrentStance = stance;
	    end
	);
	-- Shaman Change Ghost Wolf
	Soundtrack.CustomEvents.RegisterEventScript(
	    SOUNDTRACK_SHAMAN_CHANGE,
	    "Misc",
	    "UPDATE_SHAPESHIFT_FORM",
	    8,
	    false,
	    function()
	        local class = UnitClass("player")
			local stance = GetShapeshiftForm();
			if class == "Shaman" and stance ~= 0 then
				Soundtrack.CurrentStance = stance;
				Soundtrack_Custom_PlayEvent("Misc", SOUNDTRACK_SHAMAN_CHANGE)
			end
			Soundtrack.CurrentStance = stance;
	    end
	);
	-- Warrior Change Stances
	Soundtrack.CustomEvents.RegisterEventScript(
	    SOUNDTRACK_WARRIOR_CHANGE,
	    "Misc",
	    "UPDATE_SHAPESHIFT_FORM",
	    8,
	    false,
	    function()
	        local class = UnitClass("player")
			local stance = GetShapeshiftForm();
			if class == "Warrior" and stance ~= 0 and stance ~= Soundtrack.CurrentStance then
				Soundtrack_Custom_PlayEvent("Misc", SOUNDTRACK_WARRIOR_CHANGE)
			end
			Soundtrack.CurrentStance = stance;
	    end
	);
	
	-- Duel Requested
	Soundtrack.CustomEvents.RegisterEventScript(
	    SOUNDTRACK_DUEL_REQUESTED,
	    "Misc",
	    "DUEL_REQUESTED",
	    5,
	    false,
	    function()
	        Soundtrack_Custom_PlayEvent("Misc", SOUNDTRACK_DUEL_REQUESTED)
	    end
	    );
	-- Quest Complete
	Soundtrack.CustomEvents.RegisterEventScript(
	    SOUNDTRACK_QUEST_COMPLETE,
	    "Misc",
	    "QUEST_COMPLETE",
	    6,
	    false,
	    function()
	        Soundtrack_Custom_PlayEvent("Misc", SOUNDTRACK_QUEST_COMPLETE)
	    end
	);    
	-- Stealthed
	-- Thanks to zephus67 for the code!
	Soundtrack.CustomEvents.RegisterUpdateScript(
        SOUNDTRACK_STEALTHED, 
        "Misc",
        5,
        true,
        function()
            if SNDCUSTOM_IsStealthed == nil then 
                SNDCUSTOM_IsStealthed = false 
            end
            if SNDCUSTOM_IsStealthed and not IsStealthed() then
                Soundtrack_Custom_StopEvent("Misc", SOUNDTRACK_STEALTHED)
                SNDCUSTOM_IsStealthed = false 
            elseif not SNDCUSTOM_IsStealthed and IsStealthed() then
                Soundtrack_Custom_PlayEvent("Misc", SOUNDTRACK_STEALTHED)
                SNDCUSTOM_IsStealthed = true;
            end
        end
	);
	-- LFG Complete
	-- Thanks to sgtrama!
	Soundtrack.CustomEvents.RegisterEventScript(
	    SOUNDTRACK_LFG_COMPLETE,
	    "Misc",
	    "LFG_COMPLETION_REWARD",
	    6,
	    false,
	    function()
	        Soundtrack_Custom_PlayEvent("Misc", SOUNDTRACK_LFG_COMPLETE)
	    end
	);
	-- Achievement
	Soundtrack.CustomEvents.RegisterEventScript(
	    SOUNDTRACK_ACHIEVEMENT,
	    "Misc",
	    "ACHIEVEMENT_EARNED",
	    6,
	    false,
	    function()
	        Soundtrack_Custom_PlayEvent("Misc", SOUNDTRACK_ACHIEVEMENT)
	    end
	);
	
	-- Swing
	Soundtrack.CustomEvents.RegisterEventScript(
	    SOUNDTRACK_SWING_CRIT,
	    "Misc",
	    "COMBAT_LOG_EVENT_UNFILTERED",
	    8,
	    false,
	    function()
	        if arg2 == "SWING_DAMAGE" and arg4 == UnitName("player") and arg15 == 1 then
				Soundtrack_Custom_PlayEvent("Misc", SOUNDTRACK_SWING_CRIT)
			end
	    end
	);
	Soundtrack.CustomEvents.RegisterEventScript(
	    SOUNDTRACK_SWING_HIT,
	    "Misc",
	    "COMBAT_LOG_EVENT_UNFILTERED",
	    8,
	    false,
	    function()
	        if arg2 == "SWING_DAMAGE" and arg4 == UnitName("player") then
				if arg15 == nil then
					Soundtrack_Custom_PlayEvent("Misc", SOUNDTRACK_SWING_HIT)
				else 
					local eventTable = Soundtrack.Events.GetTable("Misc") 
					local numTracks = table.getn(eventTable[SOUNDTRACK_SWING_CRIT].tracks)
					if numTracks == 0 then
						Soundtrack_Custom_PlayEvent("Misc", SOUNDTRACK_SWING_HIT)
					end
				end
			end
	    end
	);

	-- Damage Spells
	Soundtrack.CustomEvents.RegisterEventScript(
	    SOUNDTRACK_SPELL_CRIT,
	    "Misc",
	    "COMBAT_LOG_EVENT_UNFILTERED",
	    8,
	    false,
	    function()
	        if arg2 == "SPELL_DAMAGE" and arg4 == UnitName("player") and arg18 == 1 then
				Soundtrack_Custom_PlayEvent("Misc", SOUNDTRACK_SPELL_CRIT)
			end
	    end
	);
	Soundtrack.CustomEvents.RegisterEventScript(
	    SOUNDTRACK_SPELL_HIT,
	    "Misc",
	    "COMBAT_LOG_EVENT_UNFILTERED",
	    8,
	    false,
	    function()
	        if arg2 == "SPELL_DAMAGE" and arg4 == UnitName("player") then
				if arg18 == nil then
					Soundtrack_Custom_PlayEvent("Misc", SOUNDTRACK_SPELL_HIT)
				else
					local eventTable = Soundtrack.Events.GetTable("Misc") 
					local numTracks = table.getn(eventTable[SOUNDTRACK_SPELL_CRIT].tracks)
					if numTracks == 0 then
						Soundtrack_Custom_PlayEvent("Misc", SOUNDTRACK_SPELL_HIT)
					end
				end
			end
	    end
	);
	
	-- DoTs
	Soundtrack.CustomEvents.RegisterEventScript(
	    SOUNDTRACK_DOT_CRIT,
	    "Misc",
	    "COMBAT_LOG_EVENT_UNFILTERED",
	    8,
	    false,
	    function()
	        if arg2 == "SPELL_PERIODIC_DAMAGE" and arg4 == UnitName("player") and arg18 == 1 then
				Soundtrack_Custom_PlayEvent("Misc", SOUNDTRACK_DOT_CRIT)
			end
	    end
	);
	Soundtrack.CustomEvents.RegisterEventScript(
	    SOUNDTRACK_DOT_HIT,
	    "Misc",
	    "COMBAT_LOG_EVENT_UNFILTERED",
	    8,
	    false,
	    function()
	        if arg2 == "SPELL_PERIODIC_DAMAGE" and arg4 == UnitName("player") then
				if arg18 == nil then
					Soundtrack_Custom_PlayEvent("Misc", SOUNDTRACK_DOT_HIT)
				else
					local eventTable = Soundtrack.Events.GetTable("Misc") 
					local numTracks = table.getn(eventTable[SOUNDTRACK_DOT_CRIT].tracks)
					if numTracks == 0 then
						Soundtrack_Custom_PlayEvent("Misc", SOUNDTRACK_DOT_HIT)
					end
				end
			end
	    end
	);
	
	-- Healing Spells
	Soundtrack.CustomEvents.RegisterEventScript(
	    SOUNDTRACK_HEAL_CRIT,
	    "Misc",
	    "COMBAT_LOG_EVENT_UNFILTERED",
	    8,
	    false,
	    function()
	        if arg2 == "SPELL_HEAL" and arg4 == UnitName("player") and arg15 == 1 then 
					Soundtrack_Custom_PlayEvent("Misc", SOUNDTRACK_HEAL_CRIT)
			end
	    end
	);
	Soundtrack.CustomEvents.RegisterEventScript(
	    SOUNDTRACK_HEAL_HIT,
	    "Misc",
	    "COMBAT_LOG_EVENT_UNFILTERED",
	    8,
	    false,
	    function()
	        if arg2 == "SPELL_HEAL" and arg4 == UnitName("player") then 
				if arg15 == nil then 
					Soundtrack_Custom_PlayEvent("Misc", SOUNDTRACK_HEAL_HIT)
				else
					local eventTable = Soundtrack.Events.GetTable("Misc") 
					local numTracks = table.getn(eventTable[SOUNDTRACK_HEAL_CRIT].tracks)
					if numTracks == 0 then
						Soundtrack_Custom_PlayEvent("Misc", SOUNDTRACK_HEAL_HIT)
					end
				end
			end
	    end
	); 
    
	-- HoTs
	Soundtrack.CustomEvents.RegisterEventScript(
	    SOUNDTRACK_HOT_CRIT,
	    "Misc",
	    "COMBAT_LOG_EVENT_UNFILTERED",
	    8,
	    false,
	    function()
	        if arg2 == "SPELL_PERIODIC_HEAL" and arg4 == UnitName("player") and arg15 == 1 then 
					Soundtrack_Custom_PlayEvent("Misc", SOUNDTRACK_HOT_CRIT)
			end
	    end
	); 
	Soundtrack.CustomEvents.RegisterEventScript(
	    SOUNDTRACK_HOT_HIT,
	    "Misc",
	    "COMBAT_LOG_EVENT_UNFILTERED",
	    8,
	    false,
	    function()
	        if arg2 == "SPELL_PERIODIC_HEAL" and arg4 == UnitName("player") then 
				if arg15 == nil then 
					Soundtrack_Custom_PlayEvent("Misc", SOUNDTRACK_HOT_HIT)
				else
					local eventTable = Soundtrack.Events.GetTable("Misc") 
					local numTracks = table.getn(eventTable[SOUNDTRACK_HOT_CRIT].tracks)
					if numTracks == 0 then
						Soundtrack_Custom_PlayEvent("Misc", SOUNDTRACK_HOT_HIT)
					end
				end
			end
	    end
	); 
    
	-- Range
	Soundtrack.CustomEvents.RegisterEventScript(
	    SOUNDTRACK_RANGE_CRIT,
	    "Misc",
	    "COMBAT_LOG_EVENT_UNFILTERED",
	    8,
	    false,
	    function()
	        if arg2 == "SWING_DAMAGE" and arg4 == UnitName("player") and arg18 == 1 then
				Soundtrack_Custom_PlayEvent("Misc", SOUNDTRACK_RANGE_CRIT)
			end
	    end
	);
	Soundtrack.CustomEvents.RegisterEventScript(
	    SOUNDTRACK_RANGE_HIT,
	    "Misc",
	    "COMBAT_LOG_EVENT_UNFILTERED",
	    8,
	    false,
	    function()
	        if arg2 == "SWING_DAMAGE" and arg4 == UnitName("player") then
				if arg18 == nil then
					Soundtrack_Custom_PlayEvent("Misc", SOUNDTRACK_RANGE_HIT)
				else 
					local eventTable = Soundtrack.Events.GetTable("Misc") 
					local numTracks = table.getn(eventTable[SOUNDTRACK_RANGE_CRIT].tracks)
					if numTracks == 0 then
						Soundtrack_Custom_PlayEvent("Misc", SOUNDTRACK_RANGE_HIT)
					end
				end
			end
	    end
	);


   
    -- Ability_Rogue_Sprint
	Soundtrack.CustomEvents.RegisterBuffEvent(SOUNDTRACK_ROGUE_SPRINT, "Misc", "Interface\\Icons\\Ability_Rogue_Sprint", 8, true)
	--[[
	Soundtrack.CustomEvents.RegisterBuffEvent("Rogue Stealth", "Misc", "Interface\\Icons\\Ability_Stealth", 5, true)
	Soundtrack.CustomEvents.RegisterBuffEvent("Druid Cat Form Ambush", "Misc", "Interface\\Icons\\Ability_Ambush", 5, true)
	--]]
	
	Soundtrack.CustomEvents.RegisterBuffEvent(SOUNDTRACK_DRUID_DASH, "Misc", "Interface\\Icons\\Ability_Druid_Dash", 8, true)
	Soundtrack.CustomEvents.RegisterBuffEvent(SOUNDTRACK_DRUID_CAT, "Misc", "Interface\\Icons\\Ability_Druid_CatForm", 8, true)
	Soundtrack.CustomEvents.RegisterBuffEvent(SOUNDTRACK_DRUID_AQUATIC, "Misc", "Interface\\Icons\\Ability_Druid_AquaticForm", 8, true)
	Soundtrack.CustomEvents.RegisterBuffEvent(SOUNDTRACK_DRUID_BEAR, "Misc", "Interface\\Icons\\Ability_Racial_BearForm", 8, true)
	Soundtrack.CustomEvents.RegisterBuffEvent(SOUNDTRACK_DRUID_TRAVEL, "Misc", "Interface\\Icons\\Ability_Druid_TravelForm", 8, true)
	Soundtrack.CustomEvents.RegisterBuffEvent(SOUNDTRACK_DRUID_FLIGHT, "Misc", "Interface\\Icons\\Ability_Druid_FlightForm", 8, true)
	
	Soundtrack.CustomEvents.RegisterBuffEvent(SOUNDTRACK_SHAMAN_GHOST_WOLF, "Misc", "Interface\\Icons\\Spell_Nature_SpiritWolf", 8, true)
	
	-- Moonkin Form
	Soundtrack.CustomEvents.RegisterUpdateScript(
	    SOUNDTRACK_DRUID_MOONKIN, "Misc", 8, true,
	    function()
	        local class = UnitClass("player")
			local buff = Soundtrack.IsBuffActive("Interface\\Icons\\Spell_Nature_ForceOfNature")
			if class == "Druid" and buff == 24858 then
				Soundtrack.MoonkinForm = true;
					
				Soundtrack_Custom_PlayEvent("Misc", SOUNDTRACK_DRUID_MOONKIN)
			elseif not Soundtrack.TreeForm then
				Soundtrack.MoonkinForm = false;
				Soundtrack_Custom_StopEvent("Misc", SOUNDTRACK_DRUID_MOONKIN)
			end
	    end
	); 
	-- Tree of Life Form
	Soundtrack.CustomEvents.RegisterUpdateScript(
	    SOUNDTRACK_DRUID_TREE, "Misc", 8, true,
	    function()
	        local class = UnitClass("player")
			local buff = Soundtrack.IsBuffActive("Interface\\Icons\\Ability_Druid_TreeofLife")
			if class == "Druid" and buff == 34123 then
				Soundtrack.TreeForm = true;
				Soundtrack_Custom_PlayEvent("Misc", SOUNDTRACK_DRUID_TREE)
			elseif not Soundtrack.MoonkinForm then
				Soundtrack.TreeForm = false;
				Soundtrack_Custom_StopEvent("Misc", SOUNDTRACK_DRUID_TREE)
			end
	    end
	);
	
	-- Make sure there are events for each customevent
	for k,v in pairs(Soundtrack_CustomEvents) do
	    Soundtrack.AddEvent("Custom", k, v.priority, v.continuous)
		if v.eventtype == "Event Script" then
			this:RegisterEvent(v.trigger)
		end
	end
end


-- nil triggers are called at each update.
function Soundtrack.CustomEvents.OnUpdate()    
       
    if Soundtrack.Settings.EnableCustomMusic then
        for k,v in pairs(Soundtrack_CustomEvents) do
            if v.eventtype == "Update Script" then
				local hasTracks = Soundtrack_Custom_HasTracks("Custom", k)
				if hasTracks then
					RunScript(v.script);
				end
            end
        end
    end

    if Soundtrack.Settings.EnableMiscMusic then
        for k,v in pairs(Soundtrack_MiscEvents) do
            if v.eventtype == "Update Script" then
				local hasTracks = Soundtrack_Custom_HasTracks("Misc", k)
				if hasTracks then
					v.script();
				end
            end
        end
    end
end

function Soundtrack.CustomEvents.OnEvent(arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, arg13, arg14, arg15, arg16, arg17, arg18, arg19, arg20)
	-- All arguments are for Combat events, which can take up to 20 args.
    if event == "AUCTION_HOUSE_SHOW" then
        Soundtrack.AuctionHouse = true
    elseif event == "AUCTION_HOUSE_CLOSED" then
        Soundtrack.AuctionHouse = false
    elseif event == "BANKFRAME_OPENED" then
        Soundtrack.Bank = true
    elseif event == "BANKFRAME_CLOSED" then
        Soundtrack.Bank = false
    elseif event == "MERCHANT_SHOW" then
        Soundtrack.Merchant = true
    elseif event == "MERCHANT_CLOSED" then
        Soundtrack.Merchant = false
    
    -- Handle custom buff events
    elseif event == "UNIT_AURA" then  -- PLAYER_AURAS_CHANGED replaced by UNIT_AURA
		if arg1 == "player" then
			if Soundtrack.Settings.EnableCustomMusic then
				for k,v in pairs(Soundtrack_CustomEvents) do
					if v.eventtype == "Buff" then
						local buffActive = Soundtrack.IsBuffActive(v.buffTexture)
						local hasTracks = Soundtrack_Custom_HasTracks("Custom", k)
						if not v.active and buffActive then
							v.active = true
							if hasTracks then
								Soundtrack_Custom_PlayEvent("Custom", k)
							end
						elseif v.active and not buffActive then
							v.active = false
							if hasTracks then
								Soundtrack_Custom_StopEvent("Custom", k)
							end
						end
					end
					if v.eventtype == "Debuff" then
						local debuffActive = Soundtrack.IsDebuffActive(v.debuffTexture)
						local hasTracks = Soundtrack_Custom_HasTracks("Custom", k)
						if not v.active and debuffActive then
							v.active = true
							if hasTracks then
								Soundtrack_Custom_PlayEvent("Custom", k)
							end
						elseif v.active and not debuffActive then
							v.active = false
							if hasTracks then
								Soundtrack_Custom_StopEvent("Custom", k)
							end
						end
					end
				end
			end
        
			if Soundtrack.Settings.EnableMiscMusic then
				for k,v in pairs(Soundtrack_MiscEvents) do
					if v.eventtype == "Buff" then
						local buffActive = Soundtrack.IsBuffActive(v.buffTexture)
						local hasTracks = Soundtrack_Custom_HasTracks("Misc", k)
						if not v.active and buffActive then
							v.active = true
							if hasTracks then
								Soundtrack_Custom_PlayEvent("Misc", k)
							end
						elseif v.active and not buffActive then
							v.active = false
							if hasTracks then
								Soundtrack_Custom_StopEvent("Misc", k)
							end
						end
					end
					if v.eventtype == "Debuff" then
						local debuffActive = Soundtrack.IsDebuffActive(v.debuffTexture)
						local hasTracks = Soundtrack_Custom_HasTracks("Misc", k)
						if not v.active and debuffActive then
							v.active = true
							if hasTracks then
								Soundtrack_Custom_PlayEvent("Misc", k)
							end
						elseif v.active and not debuffActive then
							v.active = false
							if hasTracks then
								Soundtrack_Custom_StopEvent("Misc", k)
							end
						end
					end
				end
			end
		end
    end
    
    -- Handle custom events
    if Soundtrack.Settings.EnableCustomMusic then
        for k,v in pairs(Soundtrack_CustomEvents) do
            if v.eventtype == "Event Script" then
                if event == v.trigger then
					local hasTracks = Soundtrack_Custom_HasTracks("Custom", k)
					if hasTracks then
						RunScript(v.script)
					end
                end
            end
        end
    end
    
    if Soundtrack.Settings.EnableMiscMusic then
        for k,v in pairs(Soundtrack_MiscEvents) do
            if v.eventtype == "Event Script" then
                if event == v.trigger then
					local hasTracks = Soundtrack_Custom_HasTracks("Misc", k) 
					if hasTracks then
						v.script();
					end
                end
            end
        end
    end
end